<?php

namespace App\Service;


use Illuminate\Support\Facades\Log;

class AssetServerService extends Service
{
    public function loginRule($data){
        if(empty($data['name'])) return [false, '请输入账号！'];
        if(empty($data['password'])) return [false, '请输入密码！'];

        return [true, ''];

        list($status, $msg) = $this->getToken($data);
        if(! $status) return [false, $msg];

        return [true, ['data' => $msg]];
    }

    public function getToken($data){
        $account = $data['name'];
        $password = $data['password'];

        $url = config("asset.login");
        $post = [
            "name" => $account,
            "password" => $password,
            "rememberMe" => true
        ];
        $header = ['Content-Type:application/json'];
        list($status, $result) = $this->post_helper($url,$post, $header);
        if(! $status) return [$status, $result];

        //登录失败
        if(! empty($result['errorMessage'])) return [false, $result['errorMessage']];

        return [true, $result];
    }

    public function pdList($data,$param){
        $url = config("asset.pdList");

        $post['size'] = $data['size'] ?? 10;
        $post['number'] = ($data['number'] ?? 1) - 1;

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['content'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function pdDetail($data,$param){
        if(empty($data['id'])) return [false, '请选择盘点单!'];
        $url = config("asset.pdDetail");

        $url .= $data['id'];
        list($status,$result) = $this->get_helper($url,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        return [true, $result];
    }

    public function pdUpdate($data,$param){
        if(! isset($data['tourism_inventory_a'])) return [false, '盘点单抬头数据不能为空'];
        if(! isset($data['tourism_inventory_a']['id'])) return [false, '盘点单抬头数据ID不能为空'];
        if(! isset($data['tourism_inventory_dtl_b'])) return [false, '盘点单明细数据不能为空'];

        $url = config("asset.updatePd");

        $post = [
            'tourism_inventory_a' => $data['tourism_inventory_a'],
            'tourism_inventory_dtl_b' => $data['tourism_inventory_dtl_b'],
        ];

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ''];
    }

    public function createFixedBorrow($data,$param){
        if(empty($data['id'])) return [false, 'ID不能为空'];
        if(empty($data['ssbm']) || empty($data['ssbm_show'])) return [false, '部门不能为空'];
        if(empty($data['ssfb']) || empty($data['ssfb_show'])) return [false, '分部不能为空'];
        if(empty($data['borrow_days'])) return [false, '租借天数不能为空'];
//        if(empty($data['borrow_start_date']) || empty($data['borrow_start_date_lt'])) return [false, '借用开始不能为空'];
//        if(empty($data['borrow_end_date']) || empty($data['borrow_end_date_lt'])) return [false, '借用结束不能为空'];
        list($borrow_start_date_lt,$borrow_start_date) = $this->getE8AndUTC();
        $borrow_end_date = date("Y-m-d H:i:s", strtotime("+{$data['borrow_days']} days", strtotime($borrow_start_date)));
        list($borrow_end_date_lt,$borrow_end_date) = $this->getE8AndUTC($borrow_end_date);
        if(empty($data['borrow_person']) || empty($data['borrow_person_show'])) return [false, '借用人不能为空'];

        $url = config("asset.create_fixed_borrow");

        $post = [
            'id' => $data['id'],
            'ssbm' => $data['ssbm'],
            'ssbm_show' => $data['ssbm_show'],
            'ssfb' => $data['ssfb'],
            'ssfb_show' => $data['ssfb_show'],
            'borrow_start_date' => $borrow_start_date,
            'borrow_start_date_lt' => $borrow_start_date_lt,
            'borrow_end_date' => $borrow_end_date,
            'borrow_end_date_lt' => $borrow_end_date_lt,
            'borrow_person' => $data['borrow_person'],
            'borrow_person_show' => $data['borrow_person_show'],
        ];

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ''];
    }

    public function baseFixedQuery($data,$param){
        $url = config("asset.base_fixed_query");

        if(! empty($data['id'])) {
            $post['rules'] = [
                [
                    'field' => 'id',
                    'option' => 'IN',
                    'values' => $data['id']
                ],
            ];
        }

        $post['size'] = $data['size'] ?? 10;
        $post['number'] = ($data['number'] ?? 1) - 1;

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['content'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function createFixedReturn($data,$param){
        if(empty($data['id'])) return [false, 'ID不能为空'];
        if(empty($data['ssbm']) || empty($data['ssbm_show'])) return [false, '部门不能为空'];
        if(empty($data['ssfb']) || empty($data['ssfb_show'])) return [false, '分部不能为空'];
        list($return_date_lt,$return_date) = $this->getE8AndUTC();
//        if(empty($data['return_date']) || empty($data['return_date_lt'])) return [false, '归还开始不能为空'];
        if(empty($data['return_person']) || empty($data['return_person_show'])) return [false, '归还人不能为空'];

        $url = config("asset.create_fixed_return");
        $post = [
            'id' => $data['id'],
            'ssbm' => $data['ssbm'],
            'ssbm_show' => $data['ssbm_show'],
            'ssfb' => $data['ssfb'],
            'ssfb_show' => $data['ssfb_show'],
            'return_date' => $return_date,
            'return_date_lt' => $return_date_lt,
            'return_person' => $data['return_person'],
            'return_person_show' => $data['return_person_show'],
        ];

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ''];
    }

    public function manList($data,$param){
        $url = config("asset.manList");

        $size = $data['size'] ?? 10;
        $number = ($data['number'] ?? 1) - 1;

        $url .= '&page=' . $number . '&size=' . $size;
        if(! empty($data['title'])) $url .= '&propertyNames=name&propertyValues=' . urlencode($data['title']);
        list($status,$result) = $this->get_helper($url,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        return [true, $result];
    }

    public function bmList($data,$param){
        $url = config("asset.bmList");
        $size = $data['size'] ?? 10;
        $number = ($data['number'] ?? 1) - 1;
        $url .= 'page=' . $number . '&size=' . $size;
        if(! empty($data['title'])) $url .= '&propertyNames=name&propertyValues=' . urlencode($data['title']);

        list($status,$result) = $this->get_helper($url,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        return [true, $result];
    }

    public function fbList($data,$param){
        $url = config("asset.fbList");

        if(! empty($data['id']) || ! empty($data['title'])) {
            $rules = [];
            if(! empty($data['id'])) {
                $rules[] = [
                    'field' => 'id',
                    'option' => 'IN',
                    'values' => $data['id']
                ];
            }
            if(! empty($data['title'])) {
                $rules[] = [
                    'field' => 'name',
                    'option' => 'LIKE_ANYWHERE',
                    'values' => [$data['title']]
                ];
            }
            $post['rules'] = $rules;
        }

        $post['size'] = $data['size'] ?? 10;
        $post['number'] = ($data['number'] ?? 1) - 1;

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['content'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function tourismBaseFixedBorrowList($data,$param){
        $url = config("asset.tourism_base_fixed_borrow_list");

        if(! empty($data['id']) || ! empty($data['status'])) {
            $rules = [];
            if(! empty($data['id'])) {
                $rules[] = [
                    'field' => 'base_fixed_id',
                    'option' => 'IN',
                    'values' => $data['id']
                ];
            }
            if(! empty($data['status'])) {
                $rules[] = [
                    'field' => 'status',
                    'option' => 'IN',
                    'values' => [$data['status']]
                ];
            }
            $post['rules'] = $rules;
        }

        $post['size'] = $data['size'] ?? 10;
        $post['number'] = ($data['number'] ?? 1) - 1;

        list($status,$result) = $this->post_helper($url,$post,$param['header']);
        if(! $status) return [$status, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['content'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function getE8AndUTC($dateTimeString = ""){
        $eastEightTimeZone = new \DateTimeZone('Asia/Shanghai'); // 东八区时区
        $utcTimeZone = new \DateTimeZone('UTC'); // UTC时区

        // 假设的东八区时间，没有毫秒部分
        if(! $dateTimeString) $dateTimeString = date("Y-m-d H:i:s");
        $milliseconds = '000'; // 假设的毫秒值

        // 创建DateTime对象并设置时区为东八区
        $dateTime = new \DateTime($dateTimeString, $eastEightTimeZone);

        // 将DateTime对象转换为UTC时区
        $dateTime->setTimezone($utcTimeZone);

        // 格式化DateTime对象为ISO 8601格式，并添加毫秒部分
        $formattedDate = $dateTime->format('Y-m-d\TH:i:s') . '.' . str_pad($milliseconds, 3, '0', STR_PAD_LEFT) . 'Z';

        return [$dateTimeString,$formattedDate];
    }

    public function post_helper($url, $data, $header = [], $timeout = 20){
        Log::channel('apiLog')->info('资产盘点POST', ["api" => $url , "param" => $data ,"header" => $header]);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch,  CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_ENCODING, '');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);

        if(!is_null($data)) curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        $r = curl_exec($ch);

        if ($r === false) {
            // 获取错误号
            $errorNumber = curl_errno($ch);
            // 获取错误信息
            $errorMessage = curl_error($ch);
            $message = "cURL Error #{$errorNumber}: {$errorMessage}";

            Log::channel('apiLog')->info('资产盘点POST结果', ["message" => $message]);
            return [false, $message];
        }
        curl_close($ch);

        $return = json_decode($r, true);
        Log::channel('apiLog')->info('资产盘点POST结果', ["message" => $return]);

        if(! empty($return['message']) && $return['message'] == 'error.unAuthorized') return [0, '登录凭证已失效或者不正确'];

        return [true, $return];
    }

    public function get_helper($url,$header=[],$timeout = 20){
        Log::channel('apiLog')->info('资产盘点GET', ["api" => $url ,"header" => $header]);
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => $timeout,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_HTTPHEADER => $header,
        ));
        $r = curl_exec($ch);

        if ($r === false) {
            // 获取错误号
            $errorNumber = curl_errno($ch);
            // 获取错误信息
            $errorMessage = curl_error($ch);
            $message = "cURL Error #{$errorNumber}: {$errorMessage}";
            Log::channel('apiLog')->info('资产盘点GET', ["message" => $message]);
            return [false, $message];
        }

        curl_close($ch);

        $return = json_decode($r, true);
        Log::channel('apiLog')->info('资产盘点GET', ["message" => $return]);

        if(! empty($return['message']) && $return['message'] == 'error.unAuthorized') return [0, '登录凭证已失效或者不正常'];

        return [true, $return];
    }
}
